#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
Split xchat2 log file into daily log files suitable as input for logs2html.py.

Usage: xchatlogsplit.py filename

XXX code is ugly

This is more of an example than a real script, although I have used it to
restore some actual IRC chat log history from my xchat logs.
"""

from __future__ import print_function

import locale
import os
import re
import sys
import time
from warnings import warn


STAMP_RX = re.compile(r'^[*][*][*][*] ((BEGIN|ENDING) LOGGING AT|(LOGINIMAS|ŽURNALAS) (PRADĖTAS|BAIGTAS)) ')


def readxchatlogs(infile):
    date = None
    ymd = None
    for line in infile:
        m = STAMP_RX.match(line)
        if m:
            stamp = line[len(m.group(0)):].strip()
            try:
                t = time.strptime(stamp, '%a %b %d %H:%M:%S %Y')
            except ValueError:
                locale.setlocale(locale.LC_TIME, "")
                t = time.strptime(stamp, '%a %b %d %H:%M:%S %Y')
                locale.setlocale(locale.LC_TIME, "C")

            ymd = t[:3]
            date = time.strftime("%Y-%m-%d", t)
        elif line.strip():
            assert date, 'what year?  got only %s' % line
            try:
                t = time.strptime(line[:len('Ddd YY HH:MM:SS'):], '%b %d %H:%M:%S')
            except ValueError:
                locale.setlocale(locale.LC_TIME, "")
                try:
                    t = time.strptime(stamp, '%a %b %d %H:%M:%S %Y')
                except ValueError:
                    warn("Skipping %s" % line.strip())
                    locale.setlocale(locale.LC_TIME, "C")
                    continue
                locale.setlocale(locale.LC_TIME, "C")
            t = (ymd[0], ) + t[1:]
            if t[:3] < ymd: # new year wraparound
                warn("Guessing that wraparound occurred: %s -> %s" % (ymd, t[:3]))
                t = (ymd[0] + 1, ) + t[1:]
            ymd = t[:3]
            date = time.strftime("%Y-%m-%d", t)
            line = line[len('Ddd YY '):]
        elif not date:
            continue

        assert date
        yield date, line


def main(argv=sys.argv):
    if len(argv) < 2:
        sys.exit(__doc__)
    filename = argv[1]
    prefix = os.path.basename(filename).split('-')[1].split('.')[0]
    dir = os.path.dirname(filename)
    prefix = os.path.join(dir, prefix)
    curdate = None
    outfile = None
    with open(filename) as fp:
        for date, line in readxchatlogs(fp):
            if curdate != date:
                if outfile:
                    outfile.close()
                curdate = date
                outfilename = prefix + "." + date + ".log"
                if os.path.exists(outfilename):
                    sys.exit("refusing to overwrite %s" % outfilename)
                outfile = open(outfilename, "a")
            print(line, end=' ', file=outfile)
    if outfile:
        outfile.close()


if __name__ == '__main__':
    main()



from __future__ import print_function

import doctest
import io
import os
import shutil
import sys
import tempfile
import unittest

from irclog2html.irclog2html import (
    COLOURS,
    AbstractStyle,
    ColourChooser,
    LogParser,
    MediaWikiStyle,
    NickColourizer,
    SimpleTableStyle,
    SimpleTextStyle,
    TableStyle,
    TextStyle,
    XHTMLStyle,
    XHTMLTableStyle,
    main,
    parse_args,
)


try:
    unicode
except NameError:
    # Python 3.x
    unicode = str


here = os.path.dirname(__file__)


def myrepr(o):
    """Repr that drops u prefixes on unicode strings."""
    if isinstance(o, tuple):
        if len(o) == 1:
            return '(%s, )' % ', '.join(map(myrepr, o))
        else:
            return '(%s)' % ', '.join(map(myrepr, o))
    elif isinstance(o, unicode):
        return repr(o).lstrip('u')
    else:
        return repr(o)


def doctest_LogParser():
    r"""Tests for LogParser

    I'll define a helper function to test parsing.

        >>> def test(line):
        ...     for time, what, info in LogParser([line]):
        ...         print(myrepr(time), what, myrepr(info))

    LogParser ignores empty lines

        >>> test('')
        >>> test('\n')
        >>> test('\r\n')

    All other lines result in a tuple (time, what, info)

        >>> test('14:18 * mg says Hello')
        '14:18' ACTION '* mg says Hello'

    Usually `info` is a string, but sometimes it is a tuple

        >>> test('14:18 <mg> Hello!')
        '14:18' COMMENT ('mg', 'Hello!')

    Newline characters are stripped from the line, if they are present

        >>> test('14:18 * mg says Hello\n')
        '14:18' ACTION '* mg says Hello'
        >>> test('14:18 * mg says Hello\r\n')
        '14:18' ACTION '* mg says Hello'
        >>> test('14:18 * mg says Hello\r')
        '14:18' ACTION '* mg says Hello'

    If there is no timestamp on the line, LogParser returns None

        >>> test('* mg says Hello')
        None ACTION '* mg says Hello'

    Several timestamp formats are recognized

        >>> test('14:18 <mg> Hello!')
        '14:18' COMMENT ('mg', 'Hello!')
        >>> test('[14:18] <mg> Hello!')
        '14:18' COMMENT ('mg', 'Hello!')
        >>> test('[14:18:55] <mg> Hello!')
        '14:18:55' COMMENT ('mg', 'Hello!')
        >>> test('[2004-02-04T14:18:55] <mg> Hello!')
        '2004-02-04T14:18:55' COMMENT ('mg', 'Hello!')
        >>> test('[02-Feb-2004 14:18:55] <mg> Hello!')
        '02-Feb-2004 14:18:55' COMMENT ('mg', 'Hello!')
        >>> test('[15 Jan 08:42] <mg> +++Hello+++')
        '15 Jan 08:42' COMMENT ('mg', '+++Hello+++')
        >>> test('1075917300 <mg> Hello!')
        '2004-02-04T17:55:00' COMMENT ('mg', 'Hello!')

    Excessive metainformation is stripped from nicknames

        >>> test('[15 Jan 08:42] <jsmith!n=jsmith@10.20.30.40> Hello!')
        '15 Jan 08:42' COMMENT ('jsmith', 'Hello!')

    Greater-than signs do not confuse the parser (any more)

        >>> test('14:18 <mg> apples > oranges')
        '14:18' COMMENT ('mg', 'apples > oranges')

        >>> test('[15 Jan 08:42] <jsmith!n=jsmith@10.20.30.40> Hello > world!')
        '15 Jan 08:42' COMMENT ('jsmith', 'Hello > world!')

    `what` can be COMMENT...

        >>> test('<nick> text')
        None COMMENT ('nick', 'text')

    ...ACTION...

        >>> test('* nick text')
        None ACTION '* nick text'
        >>> test('*\tnick text')
        None ACTION '*\tnick text'

    ...JOIN...

        >>> test('*** someone joined #channel')
        None JOIN '*** someone joined #channel'
        >>> test('--> someone joined')
        None JOIN '--> someone joined'
        >>> test('-!- someone has joined #channel')
        None JOIN '-!- someone has joined #channel'

    ...PART...

        >>> test('*** someone quit')
        None PART '*** someone quit'
        >>> test('<-- someone left #channel')
        None PART '<-- someone left #channel'
        >>> test('-!- someone has quit #channel')
        None PART '-!- someone has quit #channel'

    ...NICKCHANGE...

        >>> test('*** X is now known as Y')
        None NICKCHANGE ('*** X is now known as Y', 'X', 'Y')
        >>> test('--- X are now known as Y')
        None NICKCHANGE ('--- X are now known as Y', 'X', 'Y')
        >>> test('-!- X is now known as Y')
        None NICKCHANGE ('-!- X is now known as Y', 'X', 'Y')

    ...SERVER...

        >>> test('--- welcome to irc.example.org')
        None SERVER '--- welcome to irc.example.org'
        >>> test('*** welcome to irc.example.org')
        None SERVER '*** welcome to irc.example.org'
        >>> test('-!- welcome to irc.example.org')
        None SERVER '-!- welcome to irc.example.org'

    All unrecognized lines are reported as OTHER

        >>> test('what is this line doing in my IRC log file?')
        None OTHER 'what is this line doing in my IRC log file?'

    """


def doctest_LogParser_dircproxy_support():
    r"""Tests for LogParser

    I'll define a helper function to test parsing.

        >>> def test(line):
        ...     for time, what, info in LogParser([line], dircproxy=True):
        ...         print(myrepr(time), what, myrepr(info))

        >>> test('[15 Jan 08:42] <mg!n=user@10.0.0.1> -hmm')
        '15 Jan 08:42' COMMENT ('mg', 'hmm')
        >>> test('[15 Jan 08:42] <mg!n=user@10.0.0.1> +this')
        '15 Jan 08:42' COMMENT ('mg', 'this')
        >>> test('[15 Jan 08:42] <mg!n=user@10.0.0.1> maybe')
        '15 Jan 08:42' COMMENT ('mg', 'maybe')
        >>> test('[15 Jan 08:42] <mg!n=user@10.0.0.1> --1')
        '15 Jan 08:42' COMMENT ('mg', '-1')
        >>> test('[15 Jan 08:42] <mg!n=user@10.0.0.1> ++2')
        '15 Jan 08:42' COMMENT ('mg', '+2')
        >>> test('[15 Jan 08:42] <mg!n=user@10.0.0.1> +-3')
        '15 Jan 08:42' COMMENT ('mg', '-3')

    """


def doctest_LogParser_encodings():
    r"""Tests for LogParser

    I'll define a helper function to test parsing.

        >>> def test(line):
        ...     for time, what, info in LogParser([line]):
        ...         print(myrepr(time), what, myrepr(info))

    We accept input that's in UTF-8

        >>> test(b'14:18 <mg> UTF-8: \xc4\x85')
        '14:18' COMMENT ('mg', 'UTF-8: \u0105')

    and fall back to Latin-1 (actually, Windows-1252)

        >>> test(b'14:18 <mg> cp1252: \x9a')
        '14:18' COMMENT ('mg', 'cp1252: \u0161')

    For convenience, Unicode is also accepted

        >>> test(u'14:18 <mg> hello, \u2603')
        '14:18' COMMENT ('mg', 'hello, \u2603')

    """


def doctest_ColourChooser():
    """Test for ColourChooser

        >>> cc = ColourChooser()
        >>> print(cc.choose(0, 0))
        #763e3e
        >>> print(cc.choose(0, 30))
        #763e3e
        >>> print(cc.choose(1, 30))
        #407a40
        >>> print(cc.choose(30, 30))
        #e47878
        >>> print(cc.choose(31, 30))
        #79e779

    """


def doctest_NickColourizer():
    """Test for NickColourizer

        >>> nc = NickColourizer()

    The API provided by NickColourizer is dict-like

        >>> print(nc['mgedmin'])
        #407a40

    Same nick gets the same colour

        >>> print(nc['mgedmin'])
        #407a40

    Different nicks get different colours

        >>> print(nc['povbot'])
        #42427e

    Nick changes keep the old color

        >>> nc.change('mgedmin', 'mg_away')
        >>> print(nc['mg_away'])
        #407a40

    Old nick gets a new color

        >>> print(nc['mgedmin'])
        #818144

    We can handle more than 30 nics

        >>> for nick in range(100):
        ...     assert nc['nick%d' % nick] != ''

    """


class BytesIOWrapper(object):
    charset = 'UTF-8'
    closed = False

    def __init__(self, stream):
        self.stream = stream

    def readable(self):
        return False

    def writable(self):
        return True

    def seekable(self):
        return False

    def flush(self):
        self.stream.flush()

    def write(self, bytestr):
        self.stream.write(bytestr.decode(self.charset))
        self.stream.flush()


def doctest_AbstractStyle_timestamp_anchor_duplicate_timestamps():
    """Test for AbstractStyle.timestamp_anchor

        >>> style = AbstractStyle(BytesIOWrapper(sys.stdout))
        >>> print(style.timestamp_anchor('14:51:30'))
        t14:51:30
        >>> print(style.timestamp_anchor('14:51:31'))
        t14:51:31
        >>> print(style.timestamp_anchor('14:51:31'))
        t14:51:31-2
        >>> print(style.timestamp_anchor('14:51:31'))
        t14:51:31-3

    """


def doctest_SimpleTextStyle():
    """Test for SimpleTextStyle

        >>> style = SimpleTextStyle(BytesIOWrapper(sys.stdout))
        >>> style.head('IRC logs of #channel for Monday, 2008-06-10')
        <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
        <html>
        <head>
            <title>IRC logs of #channel for Monday, 2008-06-10</title>
            <meta name="generator" content="irclog2html.py ... by Marius Gedminas">
            <meta name="version" content="...">
            <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
            <meta name="viewport" content="width=device-width, initial-scale=1">
        </head>
        <body text="#000000" bgcolor="#ffffff"><tt>

        >>> style.nicktext('02:24:17', 'mgedmin', 'Hello, world!', '#77ff77')
        &lt;mgedmin&gt; Hello, world!<br>

        >>> style.nicktext('02:24', 'mgedmin', 'http://google.com/ has a new favicon', '#77ff77')
        &lt;mgedmin&gt; <a href="http://google.com/" rel="nofollow">http://google.com/</a> has a new favicon<br>

        >>> style.servermsg('02:25', LogParser.ACTION, '* mgedmin hacks <&>')
        * mgedmin hacks &lt;&amp;&gt;<br>

        >>> style.servermsg('02:26:01', LogParser.PART, '* mgedmin leaves')
        * mgedmin leaves<br>

        >>> style.foot()
        <BLANKLINE>
        <br>Generated by irclog2html.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!
        </tt></body></html>

    """


def doctest_SimpleTextStyle_colourful():
    """Test for SimpleTextStyle

        >>> style = SimpleTextStyle(BytesIOWrapper(sys.stdout),
        ...     colours=dict((what, c) for name, c, what in COLOURS))
        >>> style.head('IRC logs of #channel for Monday, 2008-06-10')
        <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
        <html>
        <head>
            <title>IRC logs of #channel for Monday, 2008-06-10</title>
            <meta name="generator" content="irclog2html.py ... by Marius Gedminas">
            <meta name="version" content="...">
            <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
            <meta name="viewport" content="width=device-width, initial-scale=1">
        </head>
        <body text="#000000" bgcolor="#ffffff"><tt>

        >>> style.nicktext('02:24:17', 'mgedmin', 'Hello, world!', '#77ff77')
        &lt;mgedmin&gt; Hello, world!<br>

        >>> style.nicktext('02:24', 'mgedmin', 'http://google.com/ has a new favicon', '#77ff77')
        &lt;mgedmin&gt; <a href="http://google.com/" rel="nofollow">http://google.com/</a> has a new favicon<br>

        >>> style.servermsg('02:25', LogParser.ACTION, '* mgedmin hacks <&>')
        <font color="#CC00CC">* mgedmin hacks &lt;&amp;&gt;</font><br>

        >>> style.servermsg('02:26:01', LogParser.PART, '* mgedmin leaves')
        <font color="#000099">* mgedmin leaves</font><br>

        >>> style.foot()
        <BLANKLINE>
        <br>Generated by irclog2html.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!
        </tt></body></html>

    """


def doctest_TextStyle_colourful():
    """Test for TextStyle

        >>> style = TextStyle(BytesIOWrapper(sys.stdout),
        ...     colours=dict((what, c) for name, c, what in COLOURS))
        >>> style.head('IRC logs of #channel for Monday, 2008-06-10')
        <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
        <html>
        <head>
            <title>IRC logs of #channel for Monday, 2008-06-10</title>
            <meta name="generator" content="irclog2html.py ... by Marius Gedminas">
            <meta name="version" content="...">
            <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
            <meta name="viewport" content="width=device-width, initial-scale=1">
        </head>
        <body text="#000000" bgcolor="#ffffff"><tt>

        >>> style.nicktext('02:24:17', 'mgedmin', 'Hello, world!', '#77ff77')
        <font color="#77ff77">&lt;mgedmin&gt;</font> <font color="#000000">Hello, world!</font><br>

        >>> style.nicktext('02:24', 'mgedmin', 'http://google.com/ has a new favicon', '#77ff77')
        <font color="#77ff77">&lt;mgedmin&gt;</font> <font color="#000000"><a href="http://google.com/" rel="nofollow">http://google.com/</a> has a new favicon</font><br>

        >>> style.servermsg('02:25', LogParser.ACTION, '* mgedmin hacks <&>')
        <font color="#CC00CC">* mgedmin hacks &lt;&amp;&gt;</font><br>

        >>> style.servermsg('02:26:01', LogParser.PART, '* mgedmin leaves')
        <font color="#000099">* mgedmin leaves</font><br>

        >>> style.foot()
        <BLANKLINE>
        <br>Generated by irclog2html.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!
        </tt></body></html>

    """


def doctest_SimpleTableStyle():
    """Test for SimpleTableStyle

        >>> style = SimpleTableStyle(BytesIOWrapper(sys.stdout))
        >>> style.head('IRC logs of #channel for Monday, 2008-06-10')
        <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
        <html>
        <head>
           <title>IRC logs of #channel for Monday, 2008-06-10</title>
            <meta name="generator" content="irclog2html.py ... by Marius Gedminas">
            <meta name="version" content="...">
            <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
            <meta name="viewport" content="width=device-width, initial-scale=1">
        </head>
        <body text="#000000" bgcolor="#ffffff"><tt>
        <table cellspacing=3 cellpadding=2 border=0>

        >>> style.nicktext('02:24:17', 'mgedmin', 'Hello, world!', '#77ff77')
        <tr bgcolor="#eeeeee"><th><font color="#77ff77"><tt>mgedmin</tt></font></th><td width="100%"><tt>Hello, world!</tt></td></tr>

        >>> style.nicktext('02:24', 'mgedmin', 'http://google.com/ has a new favicon', '#77ff77')
        <tr bgcolor="#eeeeee"><th><font color="#77ff77"><tt>mgedmin</tt></font></th><td width="100%"><tt><a href="http://google.com/" rel="nofollow">http://google.com/</a> has a new favicon</tt></td></tr>

        >>> style.servermsg('02:25', LogParser.ACTION, '* mgedmin hacks <&>')
        <tr><td colspan=2><tt>* mgedmin hacks &lt;&amp;&gt;</tt></td></tr>

        >>> style.servermsg('02:26:01', LogParser.PART, '* mgedmin leaves')
        <tr><td colspan=2><tt>* mgedmin leaves</tt></td></tr>

        >>> style.foot()
        </table>
        <BLANKLINE>
        <br>Generated by irclog2html.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!
        </tt></body></html>

    """


def doctest_TableStyle():
    """Test for TableStyle

        >>> style = TableStyle(BytesIOWrapper(sys.stdout))
        >>> style.head('IRC logs of #channel for Monday, 2008-06-10')
        <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
        <html>
        <head>
           <title>IRC logs of #channel for Monday, 2008-06-10</title>
            <meta name="generator" content="irclog2html.py ... by Marius Gedminas">
            <meta name="version" content="...">
            <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
            <meta name="viewport" content="width=device-width, initial-scale=1">
        </head>
        <body text="#000000" bgcolor="#ffffff"><tt>
        <table cellspacing=3 cellpadding=2 border=0>

        >>> style.nicktext('02:24:17', 'mgedmin', 'Hello, world!', '#77ff77')
        <tr><th bgcolor="#77ff77"><font color="#ffffff"><tt>mgedmin</tt></font></th><td width="100%" bgcolor="#eeeeee"><tt><font color="#77ff77">Hello, world!</font></tt></td></tr>

        >>> style.nicktext('02:24', 'mgedmin', 'http://google.com/ has a new favicon', '#77ff77')
        <tr><th bgcolor="#77ff77"><font color="#ffffff"><tt>mgedmin</tt></font></th><td width="100%" bgcolor="#eeeeee"><tt><font color="#77ff77"><a href="http://google.com/" rel="nofollow">http://google.com/</a> has a new favicon</font></tt></td></tr>

        >>> style.servermsg('02:25', LogParser.ACTION, '* mgedmin hacks <&>')
        <tr><td colspan=2><tt>* mgedmin hacks &lt;&amp;&gt;</tt></td></tr>

        >>> style.servermsg('02:26:01', LogParser.PART, '* mgedmin leaves')
        <tr><td colspan=2><tt>* mgedmin leaves</tt></td></tr>

        >>> style.foot()
        </table>
        <BLANKLINE>
        <br>Generated by irclog2html.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!
        </tt></body></html>

    """


def doctest_XHTMLStyle():
    """Test for XHTMLStyle

        >>> style = XHTMLStyle(BytesIOWrapper(sys.stdout))
        >>> style.head('IRC logs of #channel for Monday, 2008-06-10')
        <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
                  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
        <html>
        <head>
          <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
          <meta name="viewport" content="width=device-width, initial-scale=1" />
          <title>IRC logs of #channel for Monday, 2008-06-10</title>
          <link rel="stylesheet" href="irclog.css" />
          <meta name="generator" content="irclog2html.py ... by Marius Gedminas" />
          <meta name="version" content="..." />
        </head>
        <body>
        <h1>IRC logs of #channel for Monday, 2008-06-10</h1>
        <div class="irclog">

        >>> style.nicktext('02:24:17', 'mgedmin', 'Hello, world!', '#77ff77')
        <p id="t02:24:17" class="comment"><a href="#t02:24:17" class="time">02:24</a> <span class="nick" style="color: #77ff77">&lt;mgedmin&gt;</span> <span class="text">Hello, world!</span></p>

        >>> style.nicktext('02:24', 'mgedmin', 'http://google.com/ has a new favicon', '#77ff77')
        <p id="t02:24" class="comment"><a href="#t02:24" class="time">02:24</a> <span class="nick" style="color: #77ff77">&lt;mgedmin&gt;</span> <span class="text"><a href="http://google.com/" rel="nofollow">http://google.com/</a> has a new favicon</span></p>

        >>> style.servermsg('02:25', LogParser.ACTION, '* mgedmin hacks <&>')
        <p id="t02:25" class="action"><a href="#t02:25" class="time">02:25</a> * mgedmin hacks &lt;&amp;&gt;</p>

        >>> style.servermsg('02:25', LogParser.ACTION, '* mgedmin is very fast')
        <p id="t02:25-2" class="action"><a href="#t02:25-2" class="time">02:25</a> * mgedmin is very fast</p>

        >>> style.servermsg('02:26:01', LogParser.PART, '* mgedmin leaves')
        <p id="t02:26:01" class="part"><a href="#t02:26:01" class="time">02:26</a> * mgedmin leaves</p>

        >>> style.nicktext(None, 'mgedmin', 'what time is it?', '#77ff77')
        <p class="comment"><span class="nick" style="color: #77ff77">&lt;mgedmin&gt;</span> <span class="text">what time is it?</span></p>

        >>> style.servermsg(None, LogParser.PART, '* wombat leaves')
        <p class="part">* wombat leaves</p>

        >>> style.foot()
        </div>
        <BLANKLINE>
        <div class="generatedby">
        <p>Generated by irclog2html.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!</p>
        </div>
        </body>
        </html>

    """


def doctest_XHTMLStyle_with_navigation_and_searchbox():
    """Test for XHTMLStyle

        >>> style = XHTMLStyle(BytesIOWrapper(sys.stdout))
        >>> style.head('IRC logs of #channel for Monday, 2008-06-10',
        ...            searchbox=True,
        ...            prev=('Prev', 'prev.html'),
        ...            next=('Next', None),
        ...            index=('Index', 'index.html'))
        <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
                  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
        <html>
        <head>
          <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
          <meta name="viewport" content="width=device-width, initial-scale=1" />
          <title>IRC logs of #channel for Monday, 2008-06-10</title>
          <link rel="stylesheet" href="irclog.css" />
          <meta name="generator" content="irclog2html.py ... by Marius Gedminas" />
          <meta name="version" content="..." />
        </head>
        <body>
        <h1>IRC logs of #channel for Monday, 2008-06-10</h1>
        <BLANKLINE>
        <div class="searchbox">
        <form action="search" method="get">
        <input type="text" name="q" id="searchtext" />
        <input type="submit" value="Search" id="searchbutton" />
        </form>
        </div>
        <BLANKLINE>
        <div class="navigation"> <a href="prev.html">Prev</a> <a href="index.html">Index</a> <span class="disabled">Next</span> </div>
        <div class="irclog">

    """


def doctest_XHTMLTableStyle():
    """Test for XHTMLTableStyle

        >>> style = XHTMLTableStyle(BytesIOWrapper(sys.stdout))
        >>> style.head('IRC logs of #channel for Monday, 2008-06-10')
        <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
                  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
        <html>
        <head>
          <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
          <meta name="viewport" content="width=device-width, initial-scale=1" />
          <title>IRC logs of #channel for Monday, 2008-06-10</title>
          <link rel="stylesheet" href="irclog.css" />
          <meta name="generator" content="irclog2html.py ... by Marius Gedminas" />
          <meta name="version" content="..." />
        </head>
        <body>
        <h1>IRC logs of #channel for Monday, 2008-06-10</h1>
        <table class="irclog">

        >>> style.nicktext('02:24:17', 'mgedmin', 'Hello, world!', '#77ff77')
        <tr id="t02:24:17"><th class="nick" style="background: #77ff77">mgedmin</th><td class="text" style="color: #77ff77">Hello, world!</td><td class="time"><a href="#t02:24:17" class="time">02:24</a></td></tr>

        >>> style.nicktext('02:24', 'mgedmin', 'http://google.com/ has a new favicon', '#77ff77')
        <tr id="t02:24"><th class="nick" style="background: #77ff77">mgedmin</th><td class="text" style="color: #77ff77"><a href="http://google.com/" rel="nofollow">http://google.com/</a> has a new favicon</td><td class="time"><a href="#t02:24" class="time">02:24</a></td></tr>

        >>> style.servermsg('02:25', LogParser.ACTION, '* mgedmin hacks <&>')
        <tr id="t02:25"><td class="action" colspan="2">* mgedmin hacks &lt;&amp;&gt;</td><td><a href="#t02:25" class="time">02:25</a></td></tr>

        >>> style.servermsg('02:26:01', LogParser.PART, '* mgedmin leaves')
        <tr id="t02:26:01"><td class="part" colspan="2">* mgedmin leaves</td><td><a href="#t02:26:01" class="time">02:26</a></td></tr>

        >>> style.nicktext(None, 'mgedmin', 'what time is it?', '#77ff77')
        <tr><th class="nick" style="background: #77ff77">mgedmin</th><td class="text" colspan="2" style="color: #77ff77">what time is it?</td></tr>

        >>> style.servermsg(None, LogParser.PART, '* wombat leaves')
        <tr><td class="part" colspan="3">* wombat leaves</td></tr>

        >>> style.foot()
        </table>
        <BLANKLINE>
        <div class="generatedby">
        <p>Generated by irclog2html.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!</p>
        </div>
        </body>
        </html>

    """


def doctest_MediaWikiStyle():
    r"""Tests for MediaWikiStyle

        >>> style = MediaWikiStyle(BytesIOWrapper(sys.stdout))

    The heading is trivial

        >>> style.head('IRC logs of #channel for Monday, 2008-06-10')
        {|

    We may have simple messages

        >>> style.nicktext('02:24:17', 'mgedmin', 'Hello, world!', '#77ff77')
        |- id="t02:24:17"
        ! style="background-color: #77ff77" | mgedmin
        | style="color: #77ff77" | Hello, world!
        || [[#t02:24:17|02:24]]

    Note that we don't need special markup for hyperlinks

        >>> style.nicktext('02:24', 'mgedmin', 'http://google.com/ has a new favicon', '#77ff77')
        |- id="t02:24"
        ! style="background-color: #77ff77" | mgedmin
        | style="color: #77ff77" | http://google.com/ has a new favicon
        || [[#t02:24|02:24]]

    But we ought to escape MediaWiki markup (XXX this is not currently done,
    would someone familiar with the markup please fix the code)

        >>> style.nicktext('02:24', '[|mg|]', '[!@#$%^&*()_+{};:,./<>?]', '#77ff77')
        |- id="t02:24"
        ! style="background-color: #77ff77" | [|mg|]
        | style="color: #77ff77" | [!@#$%^&amp;*()_+{};:,./&lt;&gt;?]
        || [[#t02:24|02:24]]

    The time is optional (some IRC logs don't have it)

        >>> style.nicktext(None, 'mgedmin', 'what time is it?', '#77ff77')
        |-
        | style="background-color: #77ff77" | mgedmin
        | style="color: #77ff77" colspan="2" | what time is it?

    There are other kinds of things that happen in IRC channels

        >>> style.servermsg('02:25', LogParser.ACTION, '* mgedmin hacks')
        |- id="t02:25"
        | colspan="2" | * mgedmin hacks
        || [[#t02:25|02:25]]

        >>> style.servermsg('02:26:01', LogParser.PART, '* mgedmin leaves')
        |- id="t02:26:01"
        | colspan="2" | * mgedmin leaves
        || [[#t02:26:01|02:26]]

    The time is optional (some IRC logs don't have it)

        >>> style.servermsg(None, LogParser.JOIN, '* wombat joins')
        |-
        | colspan="3" | * wombat joins

    Again, we ought to escape MediaWiki markup (XXX this is not currently
    done, would someone familiar with the markup please fix the code)

        >>> style.servermsg(None, LogParser.SERVER, '[!@#$%^&*()_+{};:,./<>?]')
        |-
        | colspan="3" | [!@#$%^&amp;*()_+{};:,./&lt;&gt;?]

    The footer is also simple

        >>> style.foot()
        |}
        <BLANKLINE>
        Generated by irclog2html.py ... by [mailto:marius@pov.lt Marius Gedminas] - find it at [https://mg.pov.lt/irclog2html/ https://mg.pov.lt/irclog2html/]!

    """


def run(*args):
    stderr = sys.stderr
    try:
        sys.stderr = sys.stdout
        main(['irclog2html'] + list(args))
    except SystemExit as e:
        if e.args[0] != 0:
            print("SystemExit(%s)" % myrepr(e.args[0]))
    finally:
        sys.stderr = stderr


def doctest_main_can_show_help():
    """Test for main

        >>> run('--help')
        Usage: irclog2html [options] filename [...]
        <BLANKLINE>
        Colourises and converts IRC logs to HTML format for easy web reading.
        <BLANKLINE>
        Options:
          --version             show program's version number and exit
          -h, --help            show this help message and exit
        ...
          --color-action=COLOUR_ACTION, --colour-action=COLOUR_ACTION
                                select action colour (default: #CC00CC)

    """


def doctest_main_can_list_styles():
    """Test for main

        >>> run('--style', 'help')
        The following styles are available for use with irclog2html.py:
        <BLANKLINE>
          simplett
            Text style with little use of colour
        ...

    """


def doctest_main_can_handle_unknown_style():
    """Test for main

        >>> run('--style', 'nosuchstyle')
        Usage: irclog2html [options] filename [...]
        <BLANKLINE>
        irclog2html: error: unknown style: nosuchstyle
        SystemExit(2)

    """


def doctest_main_can_read_config_file():
    """Test for main

        >>> fn = os.path.join(here, 'sample.cfg')
        >>> parser, options, args = parse_args(['irclog2html', '--config', fn])
        >>> options.title
        'IRC logs for #mychannel'
        >>> options.searchbox
        True

    """


def doctest_main_can_handle_input_errors():
    """Test for main

        >>> run('/no/such/file')
        SystemExit("irclog2html: cannot open /no/such/file for reading: [Errno 2] No such file or directory: '/no/such/file'")

    """


def doctest_main():
    """Test for main

        >>> tmpdir = tempfile.mkdtemp(prefix='irclog2html-test-')
        >>> fn = os.path.join(tmpdir, 'sample.log')
        >>> _ = shutil.copyfile(os.path.join(here, 'sample.log'), fn)
        >>> run(fn)
        >>> with io.open(fn + '.html', encoding='UTF-8') as f:
        ...     print(f.read())
        <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
                  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
        ...
        >>> shutil.rmtree(tmpdir)

    """


def doctest_main_can_handle_output_errors():
    """Test for main

        >>> tmpdir = tempfile.mkdtemp(prefix='irclog2html-test-')
        >>> _ = shutil.copyfile(os.path.join(here, 'sample.log'),
        ...                     os.path.join(tmpdir, 'sample.log'))
        >>> outfilename = os.path.join(tmpdir, 'sample.log.html')
        >>> open(outfilename, 'w').close()
        >>> os.chmod(outfilename, 0o444)
        >>> run(os.path.join(tmpdir, 'sample.log'))
        SystemExit("irclog2html: cannot open ...sample.log.html for writing: [Errno 13] Permission denied: ...sample.log.html'")
        >>> os.chmod(outfilename, 0o644)
        >>> shutil.rmtree(tmpdir)

    """


def doctest_main_can_handle_missing_config_file():
    """Test for main

        >>> run('--config', '/no/such/file')
        Usage: irclog2html [options] filename [...]
        <BLANKLINE>
        irclog2html: error: can't read config file: [Errno 2] No such file or directory: '/no/such/file'
        SystemExit(2)

    """


def doctest_main_can_handle_missing_file():
    """Test for main

        >>> run('--style', 'tt', '--colour-action', 'red')
        Usage: irclog2html [options] filename [...]
        <BLANKLINE>
        irclog2html: error: please specify a filename
        SystemExit(2)

    """


def doctest_main_complains_if_single_output_file_for_multiple_input_files_is_given():
    """Test for main

        >>> run('chan1.log', 'chan2.log', '-o', 'output.html')
        Usage: irclog2html [options] filename [...]
        <BLANKLINE>
        irclog2html: error: -o must be a directory when processing multiple files
        SystemExit(2)

    """


def doctest_main_output_file():
    """Test for main

        >>> tmpdir = tempfile.mkdtemp(prefix='irclog2html-test-')
        >>> fn = os.path.join(here, 'sample.log')
        >>> outfn = os.path.join(tmpdir, 'output.html')
        >>> run(fn, '-o', outfn)
        >>> with io.open(outfn, encoding='UTF-8') as f:
        ...     print(f.read())
        <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
                  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
        ...
        >>> shutil.rmtree(tmpdir)

    """


def doctest_main_output_directory():
    """Test for main

        >>> tmpdir = tempfile.mkdtemp(prefix='irclog2html-test-')
        >>> fn = os.path.join(here, 'sample.log')
        >>> run(fn, '-o', tmpdir)
        >>> outfn = os.path.join(tmpdir, 'sample.log.html')
        >>> with io.open(outfn, encoding='UTF-8') as f:
        ...     print(f.read())
        <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
                  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
        ...
        >>> shutil.rmtree(tmpdir)

    """


def test_suite():
    optionflags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS | doctest.REPORT_NDIFF
    return unittest.TestSuite([
        doctest.DocTestSuite('irclog2html.irclog2html'),
        doctest.DocTestSuite(optionflags=optionflags),
    ])



import cgi
import datetime
import doctest
import gzip
import os
import re
import shutil
import sys
import tempfile
import unittest
from contextlib import closing

import mock
from zope.testing import renormalizing

from irclog2html.irclogsearch import (
    LogParser,
    SearchResult,
    SearchResultFormatter,
    main,
    print_search_form,
    print_search_results,
    search_irc_logs,
    search_page,
)


try:
    unicode
except NameError:
    # Python 3.x
    unicode = str


here = os.path.dirname(__file__)


class BytesIOWrapper(object):
    charset = 'UTF-8'
    closed = False

    def __init__(self, stream):
        self.stream = stream

    def readable(self):
        return False

    def writable(self):
        return True

    def seekable(self):
        return False

    def flush(self):
        self.stream.flush()

    def write(self, bytestr):
        self.stream.write(bytestr.decode(self.charset))
        self.stream.flush()


def prepare_stdout():
    if sys.version_info[0] >= 3:
        # Make the StringIO used by doctest act like the real sys.stdout, which
        # has an underlying bytes-only output stream.  This is needed because
        # we're going to print bytes in this CGI script.
        sys.stdout.buffer = BytesIOWrapper(sys.stdout)


def myrepr(o):
    """Repr that drops u prefixes on unicode strings."""
    if isinstance(o, tuple):
        if len(o) == 1:
            return '(%s, )' % ', '.join(map(myrepr, o))
        else:
            return '(%s)' % ', '.join(map(myrepr, o))
    elif isinstance(o, unicode):
        return repr(o).lstrip('u')
    else:
        return repr(o)


def doctest_SearchResultFormatter():
    """Test for SearchResultFormatter

        >>> sys.stdout.buffer = BytesIOWrapper(sys.stdout)
        >>> srf = SearchResultFormatter(sys.stdout)
        >>> srf.print_prefix()
        <table class="irclog">

        >>> srf.print_html(SearchResult(
        ...     filename='/path/to/log', link='log.html',
        ...     date=datetime.date(2013, 3, 17), time='12:34',
        ...     event=LogParser.COMMENT, info=('mgedmin', 'hi')))
        <tr id="t12:34"><th class="nick" style="background: #407a40">mgedmin</th><td class="text" style="color: #407a40">hi</td><td class="time"><a href="log.html#t12:34" class="time">12:34</a></td></tr>

        >>> srf.print_html(SearchResult(
        ...     filename='/path/to/log', link='log.html',
        ...     date=datetime.date(2013, 3, 17), time='12:35',
        ...     event=LogParser.NICKCHANGE, info=('mgedmin is now known as mg_away', 'mgedmin', 'mg_away')))
        <tr id="t12:35"><td class="nickchange" colspan="2">mgedmin is now known as mg_away</td><td><a href="log.html#t12:35" class="time">12:35</a></td></tr>

        >>> srf.print_html(SearchResult(
        ...     filename='/path/to/log', link='log.html',
        ...     date=datetime.date(2013, 3, 17), time='12:36',
        ...     event=LogParser.ACTION, info='* mgedmin jumps up and down'))
        <tr id="t12:36"><td class="action" colspan="2">* mgedmin jumps up and down</td><td><a href="log.html#t12:36" class="time">12:36</a></td></tr>

        >>> srf.print_suffix()
        </table>

    """


def gzip_copy(src, dst):
    with open(src, 'rb') as fi:
        with closing(gzip.open(dst, 'wb')) as fo:
            shutil.copyfileobj(fi, fo)


def set_up_sample():
    tmpdir = tempfile.mkdtemp(prefix='irclog2html-test-')
    gzip_copy(os.path.join(here, 'sample.log'),
              os.path.join(tmpdir, 'sample-2013-03-17.log.gz'))
    shutil.copy(os.path.join(here, 'sample.log'),
                os.path.join(tmpdir, 'sample-2013-03-18.log'))
    return tmpdir


def clean_up_sample(tmpdir):
    shutil.rmtree(tmpdir)


def doctest_search_irc_logs():
    """Test for search_irc_logs

        >>> tmpdir = set_up_sample()
        >>> for r in search_irc_logs('seen', where=tmpdir):
        ...     print('%s %s %s %s %s' % (r.link, r.date, r.time, r.event, myrepr(r.info)))
        sample-2013-03-18.log.html 2013-03-18 2005-01-08T23:47:17 COMMENT ('mgedmin', 'seen mgedmin')
        sample-2013-03-18.log.html 2013-03-18 2005-01-08T23:47:19 COMMENT ('mgedmin', '!seen mgedmin')
        sample-2013-03-18.log.html 2013-03-18 2005-01-08T23:47:19 COMMENT ('povbot', 'mgedmin: mgedmin was last seen in #pov 2 seconds ago saying: <mgedmin> seen mgedmin')
        sample-2013-03-17.log.html 2013-03-17 2005-01-08T23:47:17 COMMENT ('mgedmin', 'seen mgedmin')
        sample-2013-03-17.log.html 2013-03-17 2005-01-08T23:47:19 COMMENT ('mgedmin', '!seen mgedmin')
        sample-2013-03-17.log.html 2013-03-17 2005-01-08T23:47:19 COMMENT ('povbot', 'mgedmin: mgedmin was last seen in #pov 2 seconds ago saying: <mgedmin> seen mgedmin')

        >>> clean_up_sample(tmpdir)

    """


def doctest_search_irc_logs_limit():
    """Test for search_irc_logs

        >>> tmpdir = set_up_sample()
        >>> for r in search_irc_logs('seen', where=tmpdir, limit=2):
        ...     print('%s %s %s %s %s' % (r.link, r.date, r.time, r.event, myrepr(r.info)))
        sample-2013-03-18.log.html 2013-03-18 2005-01-08T23:47:17 COMMENT ('mgedmin', 'seen mgedmin')
        sample-2013-03-18.log.html 2013-03-18 2005-01-08T23:47:19 COMMENT ('mgedmin', '!seen mgedmin')

        >>> clean_up_sample(tmpdir)

    """


def doctest_print_search_form():
    """Test for print_search_form

        >>> print_search_form()
        <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
                  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
        <html>
        <head>
          <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
          <meta name="viewport" content="width=device-width, initial-scale=1">
          <title>Search IRC logs</title>
          <link rel="stylesheet" href="irclog.css" />
          <meta name="generator" content="irclogsearch.py ... by Marius Gedminas" />
          <meta name="version" content="..." />
        </head>
        <body>
        <h1>Search IRC logs</h1>
        <form action="" method="get">
        <input type="text" name="q" />
        <input type="submit" />
        </form>
        <BLANKLINE>
        <div class="generatedby">
        <p>Generated by irclogsearch.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!</p>
        </div>
        </body>
        </html>

    """


def doctest_print_search_results():
    r"""Test for print_search_results

        >>> tmpdir = set_up_sample()
        >>> sys.stdout.buffer = BytesIOWrapper(sys.stdout)
        >>> fmt = print_search_results('povbot', where=tmpdir)
        <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
                  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
        <html>
        <head>
          <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
          <meta name="viewport" content="width=device-width, initial-scale=1">
          <title>Search IRC logs</title>
          <link rel="stylesheet" href="irclog.css" />
          <meta name="generator" content="irclogsearch.py ... by Marius Gedminas" />
          <meta name="version" content="..." />
        </head>
        <body>
        <h1>IRC log search results for povbot</h1>
        <form action="" method="get">
        <input type="text" name="q" value="povbot" />
        <input type="submit" />
        </form>
        <ul class="searchresults">
          <li><a href="sample-2013-03-18.log.html">2013-03-18 (Monday)</a>:
        <table class="irclog">
        <tr id="t2005-01-08T23:33:54"><td class="join" colspan="2">*** povbot has joined #pov</td><td><a href="sample-2013-03-18.log.html#t2005-01-08T23:33:54" class="time">23:33</a></td></tr>
        <tr id="t2005-01-08T23:46:35"><td class="join" colspan="2">*** povbot has joined #pov</td><td><a href="sample-2013-03-18.log.html#t2005-01-08T23:46:35" class="time">23:46</a></td></tr>
        <tr id="t2005-01-08T23:47:19"><th class="nick" style="background: #407a40">povbot</th><td class="text" style="color: #407a40">mgedmin: mgedmin was last seen in #pov 2 seconds ago saying: &lt;mgedmin&gt; seen mgedmin</td><td class="time"><a href="sample-2013-03-18.log.html#t2005-01-08T23:47:19" class="time">23:47</a></td></tr>
        <tr id="t2005-01-08T23:47:54"><th class="nick" style="background: #407a40">povbot</th><td class="text" style="color: #407a40">mgedmin: web1913, jargon, foldoc, vera, and wn responded: vera: BOT Beginning Of Tape; vera: BOT Broadcast Online TV; web1913: Bot \Bot\, n. (Zo[&quot;o]l.) See {Bots}; vera: BOT Build, Operate and Transfer (networke); vera: BOT Back On Topic (telecommunication-slang, Usenet, IRC); wn: bot n : botfly larva; typically develops inside the body of a horse or sheep or human; foldoc: bot &lt;networking, chat, (6 more messages)</td><td class="time"><a href="sample-2013-03-18.log.html#t2005-01-08T23:47:54" class="time">23:47</a></td></tr>
        </table>
          </li>
          <li><a href="sample-2013-03-17.log.html">2013-03-17 (Sunday)</a>:
        <table class="irclog">
        <tr id="t2005-01-08T23:33:54-2"><td class="join" colspan="2">*** povbot has joined #pov</td><td><a href="sample-2013-03-17.log.html#t2005-01-08T23:33:54-2" class="time">23:33</a></td></tr>
        <tr id="t2005-01-08T23:46:35-2"><td class="join" colspan="2">*** povbot has joined #pov</td><td><a href="sample-2013-03-17.log.html#t2005-01-08T23:46:35-2" class="time">23:46</a></td></tr>
        <tr id="t2005-01-08T23:47:19-2"><th class="nick" style="background: #407a40">povbot</th><td class="text" style="color: #407a40">mgedmin: mgedmin was last seen in #pov 2 seconds ago saying: &lt;mgedmin&gt; seen mgedmin</td><td class="time"><a href="sample-2013-03-17.log.html#t2005-01-08T23:47:19-2" class="time">23:47</a></td></tr>
        <tr id="t2005-01-08T23:47:54-2"><th class="nick" style="background: #407a40">povbot</th><td class="text" style="color: #407a40">mgedmin: web1913, jargon, foldoc, vera, and wn responded: vera: BOT Beginning Of Tape; vera: BOT Broadcast Online TV; web1913: Bot \Bot\, n. (Zo[&quot;o]l.) See {Bots}; vera: BOT Build, Operate and Transfer (networke); vera: BOT Back On Topic (telecommunication-slang, Usenet, IRC); wn: bot n : botfly larva; typically develops inside the body of a horse or sheep or human; foldoc: bot &lt;networking, chat, (6 more messages)</td><td class="time"><a href="sample-2013-03-17.log.html#t2005-01-08T23:47:54-2" class="time">23:47</a></td></tr>
        </table>
          </li>
        </ul>
        <p>8 matches in 2 log files with 20 lines (... seconds).</p>
        <BLANKLINE>
        <div class="generatedby">
        <p>Generated by irclogsearch.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!</p>
        </div>
        </body>
        </html>

        >>> clean_up_sample(tmpdir)

    """


def doctest_search_page():
    """Test search_page
    Let's mock the dependency functions:

        >>> patcher = mock.patch.multiple("irclog2html.irclogsearch",
        ...                               print_search_form=mock.DEFAULT,
        ...                               print_search_results=mock.DEFAULT)

        >>> values = patcher.start()

    When there is a 'q' param in the GET query, the logs are searched:

        >>> form = cgi.FieldStorage(
        ...     environ={'QUERY_STRING': 'q=123', 'HTTP_METHOD': 'GET'})
        >>> search_page("The stream", form, "/logs", "#dev*.logs")
        >>> values['print_search_results'].assert_called_once_with(
        ...     '123', logfile_pattern='#dev*.logs',
        ...     stream='The stream', where='/logs')

    When there is no query, the search form is displayed:

        >>> form = cgi.FieldStorage(environ={
        ...     'HTTP_METHOD': 'GET', 'QUERY_STRING': ''})
        >>> search_page("The stream", form, "/logs", "#dev*.logs")
        >>> values['print_search_form'].assert_called_once_with('The stream')

    Clean up:

        >>> _ = patcher.stop()

    """


def doctest_main_prints_form():
    """Test for main

        >>> os.environ.pop('QUERY_STRING', None)
        >>> prepare_stdout()
        >>> main()
        Content-Type: text/html; charset=UTF-8
        <BLANKLINE>
        <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
                  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
        <html>
        <head>
          <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
          <meta name="viewport" content="width=device-width, initial-scale=1">
          <title>Search IRC logs</title>
          <link rel="stylesheet" href="irclog.css" />
          <meta name="generator" content="irclogsearch.py ... by Marius Gedminas" />
          <meta name="version" content="..." />
        </head>
        <body>
        <h1>Search IRC logs</h1>
        <form action="" method="get">
        <input type="text" name="q" />
        <input type="submit" />
        </form>
        <BLANKLINE>
        <div class="generatedby">
        <p>Generated by irclogsearch.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!</p>
        </div>
        </body>
        </html>

    """


def doctest_main_searches():
    r"""Test for main

        >>> tmpdir = set_up_sample()
        >>> prepare_stdout()
        >>> os.environ['QUERY_STRING'] = 'q=povbot'
        >>> os.environ['IRCLOG_LOCATION'] = tmpdir
        >>> os.environ['IRCLOG_GLOB'] = '*.log'
        >>> main()
        Content-Type: text/html; charset=UTF-8
        <BLANKLINE>
        <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
                  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
        <html>
        <head>
          <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
          <meta name="viewport" content="width=device-width, initial-scale=1">
          <title>Search IRC logs</title>
          <link rel="stylesheet" href="irclog.css" />
          <meta name="generator" content="irclogsearch.py ... by Marius Gedminas" />
          <meta name="version" content="..." />
        </head>
        <body>
        <h1>IRC log search results for povbot</h1>
        <form action="" method="get">
        <input type="text" name="q" value="povbot" />
        <input type="submit" />
        </form>
        <ul class="searchresults">
          <li><a href="sample-2013-03-18.log.html">2013-03-18 (Monday)</a>:
        <table class="irclog">
        <tr id="t2005-01-08T23:33:54"><td class="join" colspan="2">*** povbot has joined #pov</td><td><a href="sample-2013-03-18.log.html#t2005-01-08T23:33:54" class="time">23:33</a></td></tr>
        <tr id="t2005-01-08T23:46:35"><td class="join" colspan="2">*** povbot has joined #pov</td><td><a href="sample-2013-03-18.log.html#t2005-01-08T23:46:35" class="time">23:46</a></td></tr>
        <tr id="t2005-01-08T23:47:19"><th class="nick" style="background: #407a40">povbot</th><td class="text" style="color: #407a40">mgedmin: mgedmin was last seen in #pov 2 seconds ago saying: &lt;mgedmin&gt; seen mgedmin</td><td class="time"><a href="sample-2013-03-18.log.html#t2005-01-08T23:47:19" class="time">23:47</a></td></tr>
        <tr id="t2005-01-08T23:47:54"><th class="nick" style="background: #407a40">povbot</th><td class="text" style="color: #407a40">mgedmin: web1913, jargon, foldoc, vera, and wn responded: vera: BOT Beginning Of Tape; vera: BOT Broadcast Online TV; web1913: Bot \Bot\, n. (Zo[&quot;o]l.) See {Bots}; vera: BOT Build, Operate and Transfer (networke); vera: BOT Back On Topic (telecommunication-slang, Usenet, IRC); wn: bot n : botfly larva; typically develops inside the body of a horse or sheep or human; foldoc: bot &lt;networking, chat, (6 more messages)</td><td class="time"><a href="sample-2013-03-18.log.html#t2005-01-08T23:47:54" class="time">23:47</a></td></tr>
        </table>
          </li>
          <li><a href="sample-2013-03-17.log.html">2013-03-17 (Sunday)</a>:
        <table class="irclog">
        <tr id="t2005-01-08T23:33:54-2"><td class="join" colspan="2">*** povbot has joined #pov</td><td><a href="sample-2013-03-17.log.html#t2005-01-08T23:33:54-2" class="time">23:33</a></td></tr>
        <tr id="t2005-01-08T23:46:35-2"><td class="join" colspan="2">*** povbot has joined #pov</td><td><a href="sample-2013-03-17.log.html#t2005-01-08T23:46:35-2" class="time">23:46</a></td></tr>
        <tr id="t2005-01-08T23:47:19-2"><th class="nick" style="background: #407a40">povbot</th><td class="text" style="color: #407a40">mgedmin: mgedmin was last seen in #pov 2 seconds ago saying: &lt;mgedmin&gt; seen mgedmin</td><td class="time"><a href="sample-2013-03-17.log.html#t2005-01-08T23:47:19-2" class="time">23:47</a></td></tr>
        <tr id="t2005-01-08T23:47:54-2"><th class="nick" style="background: #407a40">povbot</th><td class="text" style="color: #407a40">mgedmin: web1913, jargon, foldoc, vera, and wn responded: vera: BOT Beginning Of Tape; vera: BOT Broadcast Online TV; web1913: Bot \Bot\, n. (Zo[&quot;o]l.) See {Bots}; vera: BOT Build, Operate and Transfer (networke); vera: BOT Back On Topic (telecommunication-slang, Usenet, IRC); wn: bot n : botfly larva; typically develops inside the body of a horse or sheep or human; foldoc: bot &lt;networking, chat, (6 more messages)</td><td class="time"><a href="sample-2013-03-17.log.html#t2005-01-08T23:47:54-2" class="time">23:47</a></td></tr>
        </table>
          </li>
        </ul>
        <p>8 matches in 2 log files with 20 lines (... seconds).</p>
        <BLANKLINE>
        <div class="generatedby">
        <p>Generated by irclogsearch.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!</p>
        </div>
        </body>
        </html>
        >>> clean_up_sample(tmpdir)

    """


checker = None
if sys.version_info[0] == 2:
    checker = renormalizing.RENormalizing([
        (re.compile(r"^\['"), r"[b'"),
        (re.compile(r"u('.*?')"), r"\1"),
    ])


def test_suite():
    optionflags = (doctest.ELLIPSIS | doctest.REPORT_NDIFF |
                   doctest.NORMALIZE_WHITESPACE)
    return unittest.TestSuite([
        doctest.DocTestSuite(optionflags=optionflags, checker=checker),
    ])


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



#!/usr/bin/env python
"""
Convert IRC logs to HTML.

Usage: irclog2html.py filename

irclog2html will write out a colourised irc log, appending a .html
extension to the output file.

This is a Python port (+ improvements) of irclog2html.pl Version 2.1, which
was written by Jeff Waugh and is available at www.perkypants.org
"""

# Copyright (c) 2005--2014, Marius Gedminas
# Copyright (c) 2000, Jeffrey W. Waugh

# Python port:
#   Marius Gedminas <marius@pov.lt>
# Original Author:
#   Jeff Waugh <jdub@perkypants.org>
# Contributors:
#   Rick Welykochy <rick@praxis.com.au>
#   Alexander Else <aelse@uu.net>
#   Ian Weller <ianweller@gmail.com>
#
# Released under the terms of the GNU GPL
# https://www.gnu.org/copyleft/gpl.html

# Differences from the Perl version:
#   There are no hardcoded nick colour preferences for jdub, cantanker and
#   chuckd
#
#   Colours are preserver accross nick changes (irclog2html.pl tries to do
#   that, but its regexes are buggy)
#
#   irclog2html.pl interprets --colour-server #rrggbb as -s #rrggbb,
#   irclog2html.py does not have this bug
#
#   irclog2html.py understands ISO 8601 timestamps (YYYY-MM-DDTHH:MM:SS)
#
#   New options: --title, --{prev,index,next}-{url,title} and many more
#
#   New styles: xhtml, xhtmltable, mediawiki
#
#   New default style: xhtmltable
#

from __future__ import print_function, unicode_literals

import datetime
import gzip
import io
import itertools
import optparse
import os
import os.path
import re
import shlex
import shutil
import sys


try:
    from urllib import quote
except ImportError:
    from urllib.parse import quote

from ._version import __date__ as RELEASE
from ._version import __homepage__ as HOMEPAGE
from ._version import __version__ as VERSION


try:
    unicode
except NameError:
    # Python 3.x
    unicode = str


# If someone packages this for a Linux distro, they'll want to patch this to
# something like /usr/share/irclog2html/irclog.css, I imagine
CSS_FILE = os.path.join(os.path.dirname(__file__), 'irclog.css')


#
# Log parsing
#

class Enum(object):
    """Enumerated value."""

    def __init__(self, value):
        self.value = value

    def __repr__(self):
        return self.value


class LogParser(object):
    """Parse an IRC log file.

    When iterated, yields the following events:

        time, COMMENT, (nick, text)
        time, ACTION, text
        time, JOIN, text
        time, PART, text,
        time, NICKCHANGE, (text, oldnick, newnick)
        time, SERVER, text

    Text is a pure ASCII or Unicode string.
    """

    COMMENT = Enum('COMMENT')
    ACTION = Enum('ACTION')
    JOIN = Enum('JOIN')
    PART = Enum('PART')
    NICKCHANGE = Enum('NICKCHANGE')
    SERVER = Enum('SERVER')
    OTHER = Enum('OTHER')

    TIME_REGEXP = re.compile(
        r'^\[?(' # Optional [
        r'(?:\d{4}-\d{2}-\d{2}T|\d{2}-\w{3}-\d{4} |\w{3} \d{2} |\d{2} \w{3} )?' # Optional date
        r'\d\d:\d\d(:\d\d)?' # Mandatory HH:MM, optional :SS
        r')\]? +') # Optional ], mandatory space
    TIMESTAMP_REGEXP = re.compile(r'^(\d+) +')
    NICK_REGEXP = re.compile(r'^<(.*?)(!.*?)?>\s')
    DIRCPROXY_NICK_REGEXP = re.compile(r'^<(.*?)(!.*)?>\s[\+-]?')
    JOIN_REGEXP = re.compile(r'^(?:\*\*\*|-->|-!-)\s.*joined')
    PART_REGEXP = re.compile(r'^(?:\*\*\*|<--|-!-)\s.*(quit|left)')
    SERVMSG_REGEXP = re.compile(r'^(?:\*\*\*|---|-!-)\s')
    NICK_CHANGE_REGEXP = re.compile(
        r'^(?:\*\*\*|---|-!-)\s+(.*?) (?:are|is) now known as (.*)')

    def __init__(self, infile, dircproxy=False):
        self.infile = infile
        if dircproxy:
            self.NICK_REGEXP = self.DIRCPROXY_NICK_REGEXP

    @staticmethod
    def decode(s):
        """Convert 8-bit string to Unicode.

        Supports xchat's hybrid Latin/Unicode encoding, as documented here:
        http://xchat.org/encoding/
        """
        if isinstance(s, unicode):
            # Accept input that's already Unicode, for convenience
            return s
        try:
            return s.decode('UTF-8')
        except UnicodeError:
            return s.decode('cp1252', 'replace')

    def __iter__(self):
        for line in self.infile:
            line = self.decode(line).rstrip('\r\n')
            if not line:
                continue

            m = self.TIME_REGEXP.match(line)
            if m:
                time = m.group(1)
                line = line[len(m.group(0)):]
            else:
                time = None

            if time is None:
                m = self.TIMESTAMP_REGEXP.match(line)
                if m:
                    time = datetime.datetime.utcfromtimestamp(
                        int(m.group(1))).isoformat()
                    line = line[len(m.group(0)):]

            m = self.NICK_REGEXP.match(line)
            if m:
                nick = m.group(1)
                text = line[len(m.group(0)):]
                yield time, self.COMMENT, (nick, text)
            elif line.startswith('* ') or line.startswith('*\t'):
                yield time, self.ACTION, line
            elif self.JOIN_REGEXP.match(line):
                yield time, self.JOIN, line
            elif self.PART_REGEXP.match(line):
                yield time, self.PART, line
            else:
                m = self.NICK_CHANGE_REGEXP.match(line)
                if m:
                    oldnick = m.group(1)
                    newnick = m.group(2)
                    yield time, self.NICKCHANGE, (line, oldnick, newnick)
                elif self.SERVMSG_REGEXP.match(line):
                    yield time, self.SERVER, line
                else:
                    yield time, self.OTHER, line


def open_log_file(filename):
    """Open a log file for parsing."""
    # We're dealing with text here.  Why open the file in binary mode?
    # Simple: the Latin/Unicode hybrid encoding monstrosity described
    # at http://xchat.org/encoding/#hybrid.  Python doesn't support this
    # natively, so we have to do the decoding ourselves.
    if filename.endswith('.gz'):
        return gzip.open(filename, 'rb')
    else:
        return io.open(filename, 'rb')


def shorttime(time):
    """Strip date and seconds from time.

        >>> print(shorttime('12:45:17'))
        12:45
        >>> print(shorttime('12:45'))
        12:45
        >>> print(shorttime('2005-02-04T12:45'))
        12:45

    """
    if 'T' in time:
        time = time.split('T')[-1]
    if time.count(':') > 1:
        time = ':'.join(time.split(':')[:2])
    return time


#
# Colouring stuff
#

class ColourChooser:
    """Choose distinguishable colours."""

    def __init__(self, rgbmin=240, rgbmax=125, a=0.95, b=0.5):
        """Define a range of colours available for choosing.

        `rgbmin` and `rgbmax` define the outmost range of colour depth (note
        that it is allowed to have rgbmin > rgbmax).

        `rgb`, if specified, is a list of (r,g,b) values where each component
        is between 0 and 1.0.

        If `rgb` is not specified, then it is constructed as
           [(a,b,b), (b,a,b), (b,b,a), (a,a,b), (a,b,a), (b,a,a)]

        You can tune `a` and `b` for the starting and ending concentrations of
        RGB.
        """
        assert 0 <= rgbmin < 256
        assert 0 <= rgbmax < 256
        self.rgbmin = rgbmin
        self.rgbmax = rgbmax
        assert 0 <= a <= 1.0
        assert 0 <= b <= 1.0
        self.rgb = [
            (a, b, b),
            (b, a, b),
            (b, b, a),
            (a, a, b),
            (a, b, a),
            (b, a, a),
        ]

    def choose(self, i, n):
        """Choose a colour.

        `n` specifies how many different colours you want in total.
        `i` identifies a particular colour in a set of `n` distinguishable
        colours.

        Returns a string '#rrggbb'.
        """
        if n == 0:
            n = 1
        r, g, b = self.rgb[i % len(self.rgb)]
        m = self.rgbmin + (self.rgbmax - self.rgbmin) * float(n - i) / n
        r, g, b = [int(c * m) for c in [r, g, b]]
        assert 0 <= r < 256
        assert 0 <= g < 256
        assert 0 <= b < 256
        return '#%02x%02x%02x' % (r, g, b)


class NickColourizer:
    """Choose distinguishable colours for nicknames."""

    def __init__(self, maxnicks=30, colour_chooser=None):
        """Create a colour chooser for nicknames.

        If you know how many different nicks there might be, specify that
        numer as `maxnicks`.  If you don't know, don't worry.

        If you really want to, you can specify a colour chooser.  Default is
        ColourChooser().
        """
        if colour_chooser is None:
            colour_chooser = ColourChooser()
        self.colour_chooser = colour_chooser
        self.nickcount = 0
        self.maxnicks = maxnicks
        self.nick_colour = {}

    def __getitem__(self, nick):
        colour = self.nick_colour.get(nick)
        if not colour:
            self.nickcount += 1
            if self.nickcount >= self.maxnicks:
                self.maxnicks *= 2
            colour = self.colour_chooser.choose(self.nickcount, self.maxnicks)
            self.nick_colour[nick] = colour
        return colour

    def change(self, oldnick, newnick):
        if oldnick in self.nick_colour:
            self.nick_colour[newnick] = self.nick_colour.pop(oldnick)


#
# HTML
#

URL_REGEXP = re.compile(r'((http|https|ftp|gopher|news)://([.,]*([^ \'")>&.,]|&amp;))*)')


def createlinks(text):
    """Replace possible URLs with links.

        >>> print(createlinks('check out &lt;http://example.com/a?b=c&amp;c=d#e&gt;!'))
        check out &lt;<a href="http://example.com/a?b=c&amp;c=d#e" rel="nofollow">http://example.com/a?b=c&amp;c=d#e</a>&gt;!

        >>> print(createlinks('http://example.com/a,'))
        <a href="http://example.com/a" rel="nofollow">http://example.com/a</a>,

        >>> print(createlinks('http://example.com/a.'))
        <a href="http://example.com/a" rel="nofollow">http://example.com/a</a>.

        >>> print(createlinks('http://example.com/a.b'))
        <a href="http://example.com/a.b" rel="nofollow">http://example.com/a.b</a>

    """
    return URL_REGEXP.sub(r'<a href="\1" rel="nofollow">\1</a>', text)


def escape(s):
    """Replace ampersands, pointies, control characters.

        >>> print(escape('"Hello" & <world>'))
        &quot;Hello&quot; &amp; &lt;world&gt;

    Control characters (ASCII 0 to 31) are stripped away

        >>> print(escape('[%s]' % ''.join([chr(x) for x in range(32)])))
        []

    """
    s = s.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;').replace('"', '&quot;')
    return ''.join([c for c in s if ord(c) > 0x1F])


#
# Output styles
#

class AbstractStyle(object):
    """A style defines the way output is formatted.

    This is not a real class, rather it is an description of how style
    classes should be written.
    """

    name = "stylename"
    description = "Single-line description"
    charset = 'US-ASCII'

    def __init__(self, outfile, *, outfilename='', colours=None):
        """Create a text formatter for writing to outfile.

        The ``colours`` dictionary may have the following items:

        - part
        - join
        - server
        - nickchange
        - action

        """
        self.outfile = io.TextIOWrapper(outfile, encoding=self.charset,
                                        errors='xmlcharrefreplace',
                                        line_buffering=True)
        self.outfilename = os.path.basename(outfilename)
        self.colours = colours or {}
        self._anchors = set()

    def __del__(self):
        """Destructor to make sure we don't close outfile prematurely."""
        if not self.outfile.closed:
            self.outfile.flush()
            self.outfile.detach()  # don't let TextIOWrapper.__del__ close it!

    def head(self, title, prev=('', ''), index=('', ''), next=('', ''),
             searchbox=False):
        """Generate the header.

        `prev`, `index` and `next` are tuples (title, url) that comprise
        the navigation bar.
        """

    def foot(self):
        """Generate the footer."""

    def servermsg(self, time, what, line):
        """Output a generic server message.

        `time` is a string.
        `line` is not escaped.
        `what` is one of LogParser event constants (e.g. LogParser.JOIN).
        """

    def nicktext(self, time, nick, text, htmlcolour):
        """Output a comment uttered by someone.

        `time` is a string.
        `nick` and `text` are not escaped.
        `htmlcolour` is a string ('#rrggbb').
        """

    def timestamp_anchor(self, time):
        anchor = 't%s' % time
        if anchor in self._anchors:
            org_anchor = anchor
            for n in itertools.count(2):
                anchor = '%s-%d' % (org_anchor, n)
                if anchor not in self._anchors:
                    break
        self._anchors.add(anchor)
        return anchor


class SimpleTextStyle(AbstractStyle):
    """Text style with little use of colour"""

    name = "simplett"
    description = __doc__
    charset = 'iso-8859-1'

    def head(self, title, prev=None, index=None, next=None, searchbox=False):
        print("""\
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<html>
<head>
\t<title>%(title)s</title>
\t<meta name="generator" content="irclog2html.py %(VERSION)s by Marius Gedminas">
\t<meta name="version" content="%(VERSION)s - %(RELEASE)s">
\t<meta http-equiv="Content-Type" content="text/html; charset=%(charset)s">
\t<meta name="viewport" content="width=device-width, initial-scale=1">
</head>
<body text="#000000" bgcolor="#ffffff"><tt>""" % {
            'VERSION': VERSION,
            'RELEASE': RELEASE,
            'title': escape(title),
            'charset': self.charset,
        }, file=self.outfile)

    def foot(self):
        print("""
<br>Generated by irclog2html.py %(VERSION)s by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
 - find it at <a href="%(HOMEPAGE)s">%(HOMEPAGE)s</a>!
 </tt></body></html>""" % {'VERSION': VERSION,
                           'HOMEPAGE': escape(HOMEPAGE)},
              end=' ', file=self.outfile)

    def servermsg(self, time, what, text):
        text = escape(text)
        text = createlinks(text)
        colour = self.colours.get(what)
        if colour:
            text = '<font color="%s">%s</font>' % (colour, text)
        self._servermsg(text)

    def _servermsg(self, line):
        print('%s<br>' % line, file=self.outfile)

    def nicktext(self, time, nick, text, htmlcolour):
        nick = escape(nick)
        text = escape(text)
        text = createlinks(text)
        text = text.replace('  ', '&nbsp;&nbsp;')
        self._nicktext(time, nick, text, htmlcolour)

    def _nicktext(self, time, nick, text, htmlcolour):
        print('&lt;%s&gt; %s<br>' % (nick, text), file=self.outfile)


class TextStyle(SimpleTextStyle):
    """Text style using colours for each nick"""

    name = "tt"
    description = __doc__

    def _nicktext(self, time, nick, text, htmlcolour):
        print('<font color="%s">&lt;%s&gt;</font>'
              ' <font color="#000000">%s</font><br>'
              % (htmlcolour, nick, text), file=self.outfile)


class SimpleTableStyle(SimpleTextStyle):
    """Table style, without heavy use of colour"""

    name = "simpletable"

    def head(self, title, prev=None, index=None, next=None, searchbox=False):
        SimpleTextStyle.head(self, title, prev, index, next, searchbox)
        print("<table cellspacing=3 cellpadding=2 border=0>", file=self.outfile)

    def foot(self):
        print("</table>", file=self.outfile)
        SimpleTextStyle.foot(self)

    def _servermsg(self, line):
        print('<tr><td colspan=2><tt>%s</tt></td></tr>' % line,
              file=self.outfile)

    def _nicktext(self, time, nick, text, htmlcolour):
        print('<tr bgcolor="#eeeeee"><th><font color="%s">'
              '<tt>%s</tt></font></th>'
              '<td width="100%%"><tt>%s</tt></td></tr>'
              % (htmlcolour, nick, text), file=self.outfile)


class TableStyle(SimpleTableStyle):
    """Default style, using a table with bold colours"""

    name = "table"
    description = __doc__

    def _nicktext(self, time, nick, text, htmlcolour):
        print('<tr><th bgcolor="%s"><font color="#ffffff">'
              '<tt>%s</tt></font></th>'
              '<td width="100%%" bgcolor="#eeeeee"><tt><font color="%s">%s</font></tt></td></tr>'
              % (htmlcolour, nick, htmlcolour, text), file=self.outfile)


class XHTMLStyle(AbstractStyle):
    """Text style, produces XHTML that can be styled with CSS"""

    name = 'xhtml'
    description = __doc__
    charset = 'UTF-8'

    CLASSMAP = {
        LogParser.ACTION: 'action',
        LogParser.JOIN: 'join',
        LogParser.PART: 'part',
        LogParser.NICKCHANGE: 'nickchange',
        LogParser.SERVER: 'servermsg',
        LogParser.OTHER: 'other',
    }

    prefix = '<div class="irclog">'
    suffix = '</div>'

    def head(self, title, prev=('', ''), index=('', ''), next=('', ''),
             searchbox=False):
        self.prev = prev
        self.index = index
        self.next = next
        print("""\
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
          "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
<head>
  <meta http-equiv="Content-Type" content="text/html; charset=%(charset)s" />
  <meta name="viewport" content="width=device-width, initial-scale=1" />
  <title>%(title)s</title>
  <link rel="stylesheet" href="irclog.css" />
  <meta name="generator" content="irclog2html.py %(VERSION)s by Marius Gedminas" />
  <meta name="version" content="%(VERSION)s - %(RELEASE)s" />
</head>
<body>""" % {'VERSION': VERSION, 'RELEASE': RELEASE,
             'title': escape(title), 'charset': self.charset}, file=self.outfile)
        self.heading(title)
        if searchbox:
            self.searchbox()
        self.navbar(prev, index, next)
        print(self.prefix, file=self.outfile)

    def heading(self, title):
        print('<h1>%s</h1>' % escape(title), file=self.outfile)

    def link(self, url, title):
        # Intentionally not escaping title so that &entities; work
        if url:
            print('<a href="%s">%s</a>'
                  % (escape(quote(url)), title or escape(url)),
                  end=' ', file=self.outfile)
        elif title:
            print('<span class="disabled">%s</span>' % title,
                  end=' ', file=self.outfile)

    def searchbox(self):
        print("""
<div class="searchbox">
<form action="search" method="get">
<input type="text" name="q" id="searchtext" />
<input type="submit" value="Search" id="searchbutton" />
</form>
</div>
""", file=self.outfile)

    def navbar(self, prev, index, next):
        prev_title, prev_url = prev
        index_title, index_url = index
        next_title, next_url = next
        if not (prev_title or index_title or next_title or
                prev_url or index_url or next_url):
            return
        print('<div class="navigation">', end=' ', file=self.outfile)
        self.link(prev_url, prev_title)
        self.link(index_url, index_title)
        self.link(next_url, next_title)
        print('</div>', file=self.outfile)

    def foot(self):
        print(self.suffix, file=self.outfile)
        self.navbar(self.prev, self.index, self.next)
        print("""
<div class="generatedby">
<p>Generated by irclog2html.py %(VERSION)s by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
 - find it at <a href="%(HOMEPAGE)s">%(HOMEPAGE)s</a>!</p>
</div>
</body>
</html>""" % {'VERSION': VERSION,
              'HOMEPAGE': escape(HOMEPAGE)}, file=self.outfile)

    def servermsg(self, time, what, text):
        """Output a generic server message.

        `time` is a string.
        `line` is not escaped.
        `what` is one of LogParser event constants (e.g. LogParser.JOIN).
        """
        text = escape(text)
        text = createlinks(text)
        if time:
            print(
                '<p id="{anchor}" class="{css_class}">'
                '<a href="{outfilename}#{anchor}" class="time">{time}</a>'
                ' {text}</p>'.format(
                    outfilename=self.outfilename,
                    anchor=self.timestamp_anchor(time),
                    css_class=self.CLASSMAP[what],
                    time=shorttime(time),
                    text=text),
                file=self.outfile)
        else:
            print(
                '<p class="{css_class}">{text}</p>'.format(
                    css_class=self.CLASSMAP[what], text=text),
                file=self.outfile)

    def nicktext(self, time, nick, text, htmlcolour):
        """Output a comment uttered by someone.

        `time` is a string.
        `nick` and `text` are not escaped.
        `htmlcolour` is a string ('#rrggbb').
        """
        nick = escape(nick)
        text = escape(text)
        text = createlinks(text)
        text = text.replace('  ', '&nbsp;&nbsp;')
        if time:
            print(
                '<p id="{anchor}" class="comment">'
                '<a href="{outfilename}#{anchor}" class="time">{time}</a> '
                '<span class="nick" style="color: {color}">'
                '&lt;{nick}&gt;</span>'
                ' <span class="text">{text}</span></p>'.format(
                    anchor=self.timestamp_anchor(time),
                    outfilename=self.outfilename,
                    time=shorttime(time),
                    color=htmlcolour,
                    nick=nick,
                    text=text),
                file=self.outfile)
        else:
            print(
                '<p class="comment">'
                '<span class="nick" style="color: {color}">'
                '&lt;{nick}&gt;</span>'
                ' <span class="text">{text}</span></p>'.format(
                    color=htmlcolour,
                    nick=nick,
                    text=text),
                file=self.outfile)


class XHTMLTableStyle(XHTMLStyle):
    """Table style, produces XHTML that can be styled with CSS"""

    name = 'xhtmltable'
    description = __doc__

    prefix = '<table class="irclog">'
    suffix = '</table>'

    def servermsg(self, time, what, text, link=''):
        text = escape(text)
        text = createlinks(text)
        if time:
            print(
                '<tr id="{anchor}">'
                '<td class="{css_class}" colspan="2">{text}</td>'
                '<td><a href="{link}#{anchor}" class="time">{time}</a></td>'
                '</tr>'.format(
                    anchor=self.timestamp_anchor(time),
                    css_class=self.CLASSMAP[what],
                    text=text,
                    link=link or self.outfilename,
                    time=shorttime(time)),
                file=self.outfile)
        else:
            print(
                '<tr>'
                '<td class="{css_class}" colspan="3">{text}</td>'
                '</tr>'.format(
                    css_class=self.CLASSMAP[what],
                    text=text),
                file=self.outfile)

    def nicktext(self, time, nick, text, htmlcolour, link=''):
        nick = escape(nick)
        text = escape(text)
        text = createlinks(text)
        text = text.replace('  ', '&nbsp;&nbsp;')
        link = link or self.outfilename
        if time:
            print(
                '<tr id="{anchor}">'
                '<th class="nick" style="background: {color}">{nick}</th>'
                '<td class="text" style="color: {color}">{text}</td>'
                '<td class="time">'
                '<a href="{link}#{anchor}" class="time">{time}</a></td>'
                '</tr>'.format(
                    anchor=self.timestamp_anchor(time),
                    color=htmlcolour,
                    nick=nick,
                    text=text,
                    link=link,
                    time=shorttime(time)),
                file=self.outfile)
        else:
            print(
                '<tr>'
                '<th class="nick" style="background: {color}">{nick}</th>'
                '<td class="text" colspan="2" style="color: {color}">{text}</td>'
                '</tr>'.format(
                    color=htmlcolour,
                    nick=nick,
                    text=text),
                file=self.outfile)


class MediaWikiStyle(AbstractStyle):
    """Table style, produces MediaWiki syntax"""

    name = 'mediawiki'
    description = __doc__

    def head(self, title, prev=('', ''), index=('', ''), next=('', ''),
             searchbox=False):
        print('{|', file=self.outfile)

    def servermsg(self, time, what, text, link=''):
        text = escape(text)
        # no need to call createlinks, MediaWiki parses links automatically
        if time:
            displaytime = shorttime(time)
            print('|- id="t%s"\n'
                  '| colspan="2" | %s\n'
                  '|| [[#t%s|%s]]'
                  % (time, text, time, displaytime), file=self.outfile)
        else:
            print('|-\n'
                  '| colspan="3" | %s' % text, file=self.outfile)

    def nicktext(self, time, nick, text, htmlcolour, link=''):
        nick = escape(nick)
        text = escape(text)
        # no need to call createlinks, MediaWiki parses links automatically
        if time:
            displaytime = shorttime(time)
            print('|- id="t%s"\n'
                  '! style="background-color: %s" | %s\n'
                  '| style="color: %s" | %s\n'
                  '|| [[#t%s|%s]] '
                  % (time, htmlcolour, nick, htmlcolour, text,
                     time, displaytime), file=self.outfile)
        else:
            print('|-\n'
                  '| style="background-color: %s" | %s\n'
                  '| style="color: %s" colspan="2" | %s '
                  % (htmlcolour, nick, htmlcolour, text), file=self.outfile)

    def foot(self):
        print('|}\n\nGenerated by irclog2html.py %(VERSION)s '
              'by [mailto:marius@pov.lt Marius Gedminas] - '
              'find it at [%(HOMEPAGE)s %(HOMEPAGE)s]!'
              % {'VERSION': VERSION, 'HOMEPAGE': HOMEPAGE}, file=self.outfile)


#
# Main
#

# All styles
STYLES = [
    SimpleTextStyle,
    TextStyle,
    SimpleTableStyle,
    TableStyle,
    XHTMLStyle,
    XHTMLTableStyle,
    MediaWikiStyle,
]

# Customizable colours
COLOURS = [
    ("part",       "#000099", LogParser.PART),
    ("join",       "#009900", LogParser.JOIN),
    ("server",     "#009900", LogParser.SERVER),
    ("nickchange", "#009900", LogParser.NICKCHANGE),
    ("action",     "#CC00CC", LogParser.ACTION),
]


def do_config_file(option, opt_str, value, parser):
    """Read options from a config file and feed them back to optparse."""
    options = []
    try:
        with open(value) as f:
            for line in f:
                line = line.strip()
                if not line or line.startswith('#'):
                    continue
                options.extend(shlex.split(line))
        # Note: you can cause an infinite loop if you have a config file that
        # includes itself!  Well, cause a RuntimeError actually.
        parser.rargs[:0] = options
    except IOError as e:
        raise optparse.OptionValueError("can't read config file: %s" % e)


def parse_args(argv=sys.argv):
    parser = optparse.OptionParser("usage: %prog [options] filename [...]",
                                   version=VERSION,
                                   prog='irclog2html',
                                   description="Colourises and converts IRC"
                                               " logs to HTML format for easy"
                                               " web reading.")
    parser.add_option('-c', '--config', action='callback', type='str',
                      metavar='FILE', callback=do_config_file,
                      help="read options from a config file")
    parser.add_option('--dircproxy', action='store_true', default=False,
                      help="dircproxy log file support (strips leading + or - from messages; off by default)")
    parser.add_option('-s', '--style', dest="style", default="xhtmltable",
                      help="format log according to specific style"
                           " (default: xhtmltable); try -s help for a list of"
                           " available styles")
    parser.add_option('-t', '--title', dest="title", default=None,
                      help="title of the page (default: same as file name)")
    parser.add_option('--prev-title', dest="prev_title", default='',
                      help="title of the previous page (default: none)")
    parser.add_option('--prev-url', dest="prev_url", default='',
                      help="URL of the previous page (default: none)")
    parser.add_option('--index-title', dest="index_title", default='',
                      help="title of the index page (default: none)")
    parser.add_option('--index-url', dest="index_url", default='',
                      help="URL of the index page (default: none)")
    parser.add_option('--next-title', dest="next_title", default='',
                      help="title of the next page (default: none)")
    parser.add_option('--next-url', dest="next_url", default='',
                      help="URL of the next page (default: none)")
    parser.add_option('-S', '--searchbox', action="store_true", dest="searchbox",
                      default=False,
                      help="include a search box")
    parser.add_option('-o', '--output-file',
                      help="destination output file or directory"
                           " (default: <input-file-name>.html)")
    for name, default, what in COLOURS:
        parser.add_option('--color-%s' % name, '--colour-%s' % name,
                          dest="colour_%s" % name, default=default,
                          help="select %s colour (default: %s)"
                               % (name, default))
    options, args = parser.parse_args(argv[1:])
    return parser, options, args


def pick_output_filename(input_filename):
    """Pick a filename for the output file."""
    if input_filename.endswith('.gz'):
        return input_filename[:-len('.gz')] + ".html"
    else:
        return input_filename + ".html"


def main(argv=sys.argv):
    parser, options, args = parse_args(argv)
    if options.style == "help":
        print("The following styles are available for use with irclog2html.py:")
        for style in STYLES:
            print()
            print("  %s" % style.name)
            print("    %s" % style.description)
        print()
        return
    for style in STYLES:
        if style.name == options.style:
            break
    else:
        parser.error("unknown style: %s" % options.style)
    colours = {}
    for name, default, what in COLOURS:
        colours[what] = getattr(options, 'colour_%s' % name)
    if not args:
        parser.error("please specify a filename")
    title = options.title
    prev = (options.prev_title, options.prev_url)
    index = (options.index_title, options.index_url)
    next = (options.next_title, options.next_url)

    if len(args) > 1 and options.output_file and not os.path.isdir(options.output_file):
        parser.error("-o must be a directory when processing multiple files")
    for filename in args:
        try:
            infile = open_log_file(filename)
        except EnvironmentError as e:
            sys.exit("%s: cannot open %s for reading: %s"
                     % (parser.prog, filename, e))
        # Why open the output file in binary mode?  We currently handle
        # encoding in our style classes, and they have different default
        # charsets, so it's simpler to just give a binary file to the
        # style class and let it deal with all the details.
        if not options.output_file:
            outfilename = pick_output_filename(filename)
        elif os.path.isdir(options.output_file):
            outfilename = os.path.join(
                options.output_file,
                os.path.basename(pick_output_filename(filename)))
        else:
            outfilename = options.output_file
        try:
            outfile = io.open(outfilename, "wb")
        except EnvironmentError as e:
            infile.close()
            sys.exit("%s: cannot open %s for writing: %s"
                     % (parser.prog, outfilename, e))
        try:
            parser = LogParser(infile, dircproxy=options.dircproxy)
            formatter = style(outfile, outfilename=outfilename, colours=colours)
            convert_irc_log(parser, formatter, title or filename,
                            prev, index, next, searchbox=options.searchbox)
            css_file = os.path.join(os.path.dirname(outfilename), 'irclog.css')
            if not os.path.exists(css_file) and os.path.exists(CSS_FILE):
                shutil.copy(CSS_FILE, css_file)
        finally:
            outfile.close()
            infile.close()


def convert_irc_log(parser, formatter, title, prev, index, next,
                    searchbox=False):
    """Convert IRC log to HTML or some other format."""
    nick_colour = NickColourizer()
    formatter.head(title, prev, index, next, searchbox=searchbox)
    for time, what, info in parser:
        if what == LogParser.COMMENT:
            nick, text = info
            htmlcolour = nick_colour[nick]
            formatter.nicktext(time, nick, text, htmlcolour)
        else:
            if what == LogParser.NICKCHANGE:
                text, oldnick, newnick = info
                nick_colour.change(oldnick, newnick)
            else:
                text = info
            formatter.servermsg(time, what, text)
    formatter.foot()


if __name__ == '__main__':
    main()



import datetime
import doctest
import optparse
import os
import shutil
import sys
import tempfile
import time
import unittest

from irclog2html.logs2html import (
    Error,
    LogFile,
    find_log_files,
    main,
    move_symlink,
    process,
    write_index,
)


class TestCase(unittest.TestCase):

    def setUp(self):
        self.tmpdir = None
        self.start_time = time.time()

    def tearDown(self):
        if self.tmpdir:
            try:
                shutil.rmtree(self.tmpdir)
            except Exception as e:
                sys.stderr.write(
                    "\nAn error happened while cleaning up %s:\n%s: %s\n"
                    % (self.tmpdir, e.__class__.__name__, e))
                sys.stderr.flush()

    def filename(self, filename):
        if self.tmpdir is None:
            self.tmpdir = tempfile.mkdtemp(prefix='irclog2html-test-')
        return os.path.join(self.tmpdir, filename)

    def create(self, filename, mtime=None):
        fullfilename = self.filename(filename)
        with open(fullfilename, 'w'):
            pass
        if mtime:
            mtime += self.start_time
            os.utime(fullfilename, (mtime, mtime))


class TestLogFile(TestCase):

    def LogFile(self, filename):
        return LogFile(self.filename(filename))

    def test_with_date(self):
        # no need to create temporary files for this test, so don't use
        # self.LogFile
        lf = LogFile('/path/to/somechannel-20130318.log')
        self.assertEqual(lf.date, datetime.date(2013, 3, 18))
        self.assertEqual(lf.link, 'somechannel-20130318.log.html')
        self.assertEqual(lf.title, '2013-03-18 (Monday)')

    def test_without_date(self):
        # no need to create temporary files for this test, so don't use
        # self.LogFile
        self.assertRaises(Error, LogFile, '/path/to/somechannel.log')

    def test_equality(self):
        lf1 = LogFile('/path/to/somechannel-20130318.log')
        lf2 = LogFile('/path/to/somechannel-20130319.log')
        lf3 = LogFile('/path/to/somechannel-20130318.log')
        self.assertEqual(lf1, lf1)
        self.assertEqual(lf1, lf3)
        self.assertNotEqual(lf1, lf2)

    def test_newfile_html_exists(self):
        self.create('somechannel-20130317.log')
        self.create('somechannel-20130317.log.html')
        lf = self.LogFile('somechannel-20130317.log')
        self.assertFalse(lf.newfile())

    def test_newfile_html_doesnt_exist(self):
        self.create('somechannel-20130318.log')
        lf = self.LogFile('somechannel-20130318.log')
        self.assertTrue(lf.newfile())

    def test_uptodate_html_doesnt_exist(self):
        self.create('somechannel-20130317.log')
        lf = self.LogFile('somechannel-20130317.log')
        self.assertFalse(lf.uptodate())

    def test_uptodate_html_is_newer(self):
        self.create('somechannel-20130317.log', mtime=-100)
        self.create('somechannel-20130317.log.html', mtime=-50)
        lf = self.LogFile('somechannel-20130317.log')
        self.assertTrue(lf.uptodate())

    def test_uptodate_html_is_older(self):
        self.create('somechannel-20130317.log', mtime=-10)
        self.create('somechannel-20130317.log.html', mtime=-50)
        lf = self.LogFile('somechannel-20130317.log')
        self.assertFalse(lf.uptodate())

    def test_uptodate_html_is_same_age(self):
        self.create('somechannel-20130317.log', mtime=-10)
        self.create('somechannel-20130317.log.html', mtime=-10)
        lf = self.LogFile('somechannel-20130317.log')
        self.assertFalse(lf.uptodate()) # err on the safe side: regenerate

    def test_generate(self):
        self.create('somechannel-20130317.log')
        lf = self.LogFile('somechannel-20130317.log')
        lf.generate(style='tt')
        outfile = self.filename('somechannel-20130317.log.html')
        self.assertTrue(os.path.exists(outfile))

    def test_generate_with_navbar(self):
        self.create('somechannel-20130317.log')
        prev = self.LogFile('somechannel-20130316.log')
        lf = self.LogFile('somechannel-20130317.log')
        next = self.LogFile('somechannel-20130318.log')
        lf.generate(style='tt', prev=prev, next=next)
        outfile = self.filename('somechannel-20130317.log.html')
        self.assertTrue(os.path.exists(outfile))

    def test_find_log_files(self):
        self.create('somechannel-20130316.log')
        self.create('somechannel-20130316.log.html')
        self.create('somechannel-20130317.log')
        self.create('somechannel-20130318.log')
        self.assertEqual(find_log_files(self.tmpdir),
                         [self.LogFile('somechannel-20130316.log'),
                          self.LogFile('somechannel-20130317.log'),
                          self.LogFile('somechannel-20130318.log')])

    def test_move_symlink(self):
        if not hasattr(os, 'symlink'):
            if not hasattr(self, 'skipTest'): # Python 2.6
                return
            self.skipTest("platform does not support symlinks")
        move_symlink('somechannel-20130316.log.html',
                     self.filename('latest.log.html'))
        self.assertEqual(os.readlink(self.filename('latest.log.html')),
                         'somechannel-20130316.log.html')
        move_symlink('somechannel-20130317.log.html',
                     self.filename('latest.log.html'))
        self.assertEqual(os.readlink(self.filename('latest.log.html')),
                         'somechannel-20130317.log.html')

    def test_process(self):
        self.create('somechannel-20130316.log')
        self.create('somechannel-20130316.log.html')
        self.create('somechannel-20130317.log')
        self.create('somechannel-20130318.log')
        options = optparse.Values(dict(searchbox=True, dircproxy=True,
                                       pattern='*.log', force=False,
                                       prefix='IRC logs for ', output_dir=None,
                                       style='xhtmltable', title='IRC logs'))
        process(self.tmpdir, options)
        self.assertTrue(os.path.exists(self.filename('index.html')))
        if hasattr(os, 'symlink'):
            self.assertTrue(os.path.exists(self.filename('latest.log.html')))
        self.assertTrue(os.path.exists(self.filename('irclog.css')))
        self.assertTrue(os.path.exists(
            self.filename('somechannel-20130316.log.html')))
        self.assertTrue(os.path.exists(
            self.filename('somechannel-20130317.log.html')))
        self.assertTrue(os.path.exists(
            self.filename('somechannel-20130318.log.html')))

    def test_process_copies_css_even_when_all_logs_up_to_date(self):
        self.create('somechannel-20130316.log', mtime=-10)
        self.create('somechannel-20130316.log.html')
        options = optparse.Values(dict(searchbox=True, dircproxy=True,
                                       pattern='*.log', force=False,
                                       prefix='IRC logs for ', output_dir=None,
                                       style='xhtmltable', title='IRC logs'))
        process(self.tmpdir, options)
        self.assertTrue(os.path.exists(self.filename('index.html')))
        if hasattr(os, 'symlink'):
            self.assertTrue(os.path.exists(self.filename('latest.log.html')))
        self.assertTrue(os.path.exists(self.filename('irclog.css')))

    def test_process_handles_write_errors(self):
        self.create('somechannel-20130316.log', mtime=-10)
        self.create('somechannel-20130316.log.html')
        self.create('index.html')
        options = optparse.Values(dict(searchbox=True, dircproxy=True,
                                       pattern='*.log', force=False,
                                       prefix='IRC logs for ', output_dir=None,
                                       style='xhtmltable', title='IRC logs'))
        os.chmod(self.filename('index.html'), 0o444)
        self.assertRaises(Error, process, self.tmpdir, options)
        # shutil.rmtree() on Windows can't handle read-only files.
        # Restore the permissions so that our tearDown() can succeed.
        os.chmod(self.filename('index.html'), 0o644)

    def test_process_with_output_dir(self):
        self.create('somechannel-20130316.log', mtime=-200)
        self.create('somechannel-20130316.log.html', mtime=-250)  # outdated
        self.create('somechannel-20130317.log', mtime=-100)
        self.create('somechannel-20130317.log.html', mtime=-50)   # up to date
        options = optparse.Values(dict(searchbox=True, dircproxy=True,
                                       pattern='*.log', force=False,
                                       prefix='IRC logs for ',
                                       output_dir=self.filename('new/out/dir'),
                                       style='xhtmltable', title='IRC logs'))
        process(self.tmpdir, options)
        self.assertTrue(os.path.exists(self.filename('new/out/dir/index.html')))
        self.assertTrue(os.path.exists(self.filename('new/out/dir/irclog.css')))
        self.assertTrue(os.path.exists(
            self.filename('new/out/dir/somechannel-20130316.log.html')))
        self.assertTrue(os.path.exists(
            self.filename('new/out/dir/somechannel-20130317.log.html')))
        if hasattr(os, 'symlink'):
            self.assertTrue(os.path.exists(
                self.filename('new/out/dir/latest.log.html')))

    def test_process_with_output_dir_failure_to_mkdir(self):
        self.create('somechannel-20130316.log')
        self.create('out')
        options = optparse.Values(dict(searchbox=True, dircproxy=True,
                                       pattern='*.log', force=False,
                                       prefix='IRC logs for ',
                                       output_dir=self.filename('out'),
                                       style='xhtmltable', title='IRC logs'))
        with self.assertRaises(Error):
            process(self.tmpdir, options)

    def test_main(self):
        self.create('somechannel-20130316.log')
        self.create('somechannel-20130316.log.html')
        self.create('somechannel-20130317.log')
        self.create('somechannel-20130318.log')
        main(['logs2html', self.tmpdir])
        self.assertTrue(os.path.exists(self.filename('index.html')))
        if hasattr(os, 'symlink'):
            self.assertTrue(os.path.exists(self.filename('latest.log.html')))
        self.assertTrue(os.path.exists(self.filename('irclog.css')))
        self.assertTrue(os.path.exists(
            self.filename('somechannel-20130316.log.html')))
        self.assertTrue(os.path.exists(
            self.filename('somechannel-20130317.log.html')))
        self.assertTrue(os.path.exists(
            self.filename('somechannel-20130318.log.html')))

    def test_main_error_handling(self):
        self.create('nodate.log')
        self.assertRaises(SystemExit, main, ['logs2html', self.tmpdir])


def doctest_write_index():
    """Test for write_index

        >>> write_index(sys.stdout,
        ...             title='IRC logs for somechannel',
        ...             logfiles=[LogFile('somechannel-20130316.log'),
        ...                       LogFile('somechannel-20130317.log'),
        ...                       LogFile('somechannel-20130318.log')],
        ...             latest_log_link='latest.html')
        <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
                  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
        <html>
        <head>
          <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
          <meta name="viewport" content="width=device-width, initial-scale=1">
          <title>IRC logs for somechannel</title>
          <link rel="stylesheet" href="irclog.css" />
          <meta name="generator" content="logs2html.py ... by Marius Gedminas" />
          <meta name="version" content="..." />
        </head>
        <body>
        <h1>IRC logs for somechannel</h1>
        <ul>
        <li><a href="latest.html">Latest (bookmarkable)</a></li>
        </ul>
        <ul>
        </ul>
        <h2>2013-03</h2>
        <ul>
        <li><a href="somechannel-20130316.log.html">2013-03-16 (Saturday)</a></li>
        <li><a href="somechannel-20130317.log.html">2013-03-17 (Sunday)</a></li>
        <li><a href="somechannel-20130318.log.html">2013-03-18 (Monday)</a></li>
        </ul>
        <BLANKLINE>
        <div class="generatedby">
        <p>Generated by logs2html.py ... by <a href="mailto:marius@pov.lt">Marius Gedminas</a>
         - find it at <a href="https://mg.pov.lt/irclog2html/">https://mg.pov.lt/irclog2html/</a>!</p>
        </div>
        </body>
        </html>

    """


def run(*args):
    stderr = sys.stderr
    try:
        sys.stderr = sys.stdout
        main(['logs2html'] + list(args))
    except SystemExit as e:
        if e.args[0] != 0:
            print("SystemExit(%s)" % repr(e.args[0]))
    finally:
        sys.stderr = stderr


def doctest_main_can_print_help():
    """Test for main

        >>> run('--help')
        Usage: logs2html [options] directory
        <BLANKLINE>
        Colourises and converts all IRC logs to HTML format for easy web reading.
        <BLANKLINE>
        Options:
          --version             show program's version number and exit
          -h, --help            show this help message and exit
          ...

    """


def doctest_main_missing_dirname():
    """Test for main

        >>> run('--style', 'tt')
        Usage: logs2html [options] directory
        <BLANKLINE>
        logs2html: error: missing directory name
        SystemExit(2)

    """


def doctest_main_extra_args():
    """Test for main

        >>> run('dir1', 'dir2')
        Usage: logs2html [options] directory
        <BLANKLINE>
        logs2html: error: too many arguments
        SystemExit(2)

    """


def test_suite():
    return unittest.TestSuite([
        doctest.DocTestSuite(optionflags=doctest.ELLIPSIS | doctest.REPORT_NDIFF),
        unittest.makeSuite(TestLogFile),
    ])
